@@ -211,6 +211,10 @@ pub mod config {
     }
 
     impl Config {
+        pub fn new() -> Self {
+            Default::default()
+        }
+
         #[must_use]
         pub fn baudrate(mut self, baudrate: Hertz) -> Self {
             self.baudrate = baudrate;
